/* src/add_info.c - Ajout d'informations en mmoire
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "main.h"

CVSINFO("$Id: add_info.c,v 1.25 2003/05/03 16:51:46 romexzf Exp $");

extern struct badchans *badchead;
extern aKill *killhead;
extern struct badnicks *badnhead;
extern struct ignore *ignorehead;

aNick *add_nickinfo(const char *nick, const char *user, const char *host, const char *base64, const char *num, struct Link *server, const char *name, time_t ttmco)
{
	aNick *new = calloc(1, sizeof(struct nickinfo));

	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour aNick %s !\n", nick);
		return NULL;
	}
	new->next = NULL;
	strncpy(new->nick, nick, NICKLEN);
	strncpy(new->ident, user, USERLEN);
	strncpy(new->host, host, HOSTLEN);
	strncpy(new->base64, base64, 7);
	strcpy(new->numeric, num);
	new->serveur = server;
	strncpy(new->name, name, REALEN);
	new->ttmco = ttmco;
	new->floodtime = time(NULL);
	new->floodcount = 0;
	new->last = NULL;
	if(!nickhead) nickhead = new;
	else
	{
		nickhead->last = new;
		new->next = nickhead;
		nickhead = new;
	}
	return new;
}

#ifdef USE_MEMOSERV
void add_memo(anUser *nick, const char *de, time_t date, const char *message, int read)
{
	struct memos *new = calloc(1, sizeof(struct memos)), *current;

	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour aMemo %s !\n", nick->nick);
		return;
	}
	new->next = NULL;
	strncpy(new->de, de, NICKLEN);
	new->date = date;
	new->read = read;
	strncpy(new->message, message, MEMOLEN);
	if(!nick->memohead) nick->memohead = new;
	else
	{
		for(current=nick->memohead;current->next;current=current->next);
		if(!current->next) current->next = new;
		else free(new);
	}
}
#endif

struct badchans *add_abadchan(const char *chaine, const char *raison)
{
	struct badchans *new = calloc(1, sizeof(struct badchans)), *current;

	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour un badchans (%s) !\n", chaine);
		return NULL;
	}
	new->next = NULL;
	strncpy(new->chaine, chaine, 100);
	strncpy(new->raison, raison, 150);
	if(!badchead) badchead = new;
	else
	{
		for(current=badchead;current->next && match(current->chaine, new->chaine);current=current->next);
		if(!current->next) current->next=new;
		else free(new);
	}
	return new;
}

aChan *add_chan(const char *chan, const char *topic, const char *description, const char *defmodes, const char *deftopic, const char *welcome, int flag, int banlevel, int bantype, const char *motd, const char *chanurl)
{
	aChan *new = calloc(1, sizeof(struct chaninfo)), *current;

	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour aChan %s !\n", chan);
		return NULL;
	}
	new->next = NULL;
	strncpy(new->chan, chan, 100);
	strncpy(new->topic, topic, TOPICLEN);
	strcpy(new->defmodes, defmodes);
	strncpy(new->deftopic, deftopic,TOPICLEN);
	strncpy(new->description, description,150);
	strcpy(new->welcome, welcome);
	new->suspend = NULL;
	new->flag = flag;
	new->banlevel = banlevel;
	new->bantype = (bantype > 4 || bantype < 1) ? 1 : bantype;
	strncpy(new->motd, motd, 250);
	strncpy(new->url, chanurl, 100);
	new->banhead = NULL;

	if(!chanhead) chanhead = new;
	else
	{
		for(current=chanhead;current->next && strcasecmp(current->chan, chan);current=current->next);
		if(!current->next) current->next=new;
		else free(new);
	}
	return new;
}

anAccess *add_access(anUser *user, const char *chan, int level, int flag, time_t lastseen)
{
	anAccess *new = calloc(1, sizeof(struct access)), *current;
	aChan *c;
	aLink *lp;

	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour anAccess %s sur %s!\n", user->nick, chan);
		return NULL;
	}
	new->next = NULL;
	new->level = level;
	new->flag = flag;
	new->user = user;
	new->info = NULL;
	strncpy(new->chan, chan, 100);
	if((c=getchaninfo(chan))) new->c = c;
	else
	{
		free(new);
		return NULL;
	}

	if(lastseen) new->lastseen = lastseen;
	else new->lastseen = user->lastseend;

	if(!user->accesshead) user->accesshead = new;
	else
	{
		for(current=user->accesshead;current->next && strcasecmp(current->chan,chan);current=current->next);
		if(!current->next) current->next=new;
		else free(new);
	}

	lp = calloc(1, sizeof(aLink));
	lp->value.a = new;
	lp->next = c->access;
	c->access = lp;
	return new;
}

void add_killinfo(const char *nick, const char *numeric, time_t timestamp, int why, int setkill)
{
	struct killinfo *new = calloc(1, sizeof(struct killinfo)), *current;

	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour killinfo %s !\n", nick);
		return;
	}
	new->next = NULL;
	strncpy(new->nick, nick, NICKLEN);
	strcpy(new->numeric, numeric);
	new->timestamp = timestamp;
	new->why = why;
	new->setkill = setkill;
	if(!killhead) killhead = new;
	else
	{
		for(current=killhead;current->next;current=current->next);
		if(!current->next) current->next=new;
		else free(new);
	}
}

void add_abadnick(const char *chaine, int flag)
{
	struct badnicks *new = calloc(1, sizeof(struct badnicks)), *current;

	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour un badnick %s !\n", chaine);
		return;
	}
	new->next = NULL;
	strncpy(new->chaine, chaine, NICKLEN);
	new->flag = flag;
	if(!badnhead) badnhead = new;
	else
	{
		for(current = badnhead;current->next && match(current->chaine, new->chaine);current=current->next);
		if(!current->next) current->next=new;
		else free(new);
	}
}

anUser *add_regnick(const char *user, const char *pass, time_t lastseend, int level, int flag, const char *mail, int vote)
{
	anUser *new;

	if(!(new = calloc(1,sizeof(struct userinfo))))
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour anUser %s !\n", user);
		return NULL;
	}
	new->next = NULL;
	strncpy(new->nick, user, NICKLEN);
	strcpy(new->passwd, pass);
	new->lastseend = lastseend;
	new->level = level;
	strncpy(new->mail, mail, 100);
#ifdef HAVE_VOTE
	new->vote = vote;
#endif
	new->flag = flag;
	new->last = NULL;
	new->suspend = NULL;
	new->lastlogin = NULL;
	if(!userhead) userhead = new;
	else
	{
		userhead->last=new;
		new->next=userhead;
		userhead=new;
	}
	return new;
}

void add_join(aNick *nick, char *chan, int is_op, int is_voice, time_t timestamp, int is_burst)
{

	aJoin *new = calloc(1, sizeof(aJoin)), *join;
	anAccess *access;
	aChan *chaninfo = NULL;
	aLink *lp, *l;

	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour aJoin %s sur %s !\n", nick->nick, chan);
		return;
	}
	new->next = NULL;
	strcpy(new->channel, chan);
	new->is_op = is_op;
	new->is_voice = is_voice;
	new->timestmp = timestamp;
	new->nick = nick;
	if(!nick->joinhead) nick->joinhead = new;
	else
	{
		for(join = nick->joinhead;join->next;join=join->next);
		if(!join->next && strcasecmp(join->channel, chan)) join->next=new;
		else /* le mec est dja sur le chan ? */
		{
			free(new);
			new=NULL;
		}
	}

	if(!(chaninfo = getchaninfo(chan))) return;
	if(new)
	{
		lp = calloc(1, sizeof(aLink));
		lp->value.j = new;
		if(!chaninfo->members) chaninfo->members = lp;
		else
		{
			for(l = chaninfo->members;l->next;l=l->next); /* on cherche le dernier */
			l->next = lp;
		}
	}

	if(IsSuspend(chaninfo)) /* envoi d'une notice lors du create d'un salon Reg mais suspendu.*/
	{						/* mais pas lors du burst*/
		if(is_burst || !is_op) return;/*chaninfo + reg? + suspend? (kouak)*/
		csntc(nick, "Le salon %s est actullement suspendu par %s (%s)", chaninfo->chan, chaninfo->suspend->from, chaninfo->suspend->raison);
		putserv("%s %s %s -o %s\n", bot.servnum, TOKEN_MODE, chaninfo->chan, nick->numeric);
		new->is_op = 0;
		return;
	}
	if(!is_burst)
	{
		/*ban check CESARRRRRRRRRRR (mis ici pr eviter 2 loooooooops)*/
		if(!(nick->flag & N_OPER) && (!nick->user || !IsAnAdmin(nick->user)))
		{
			aBan *ban;
			if((ban = is_ban(nick, chaninfo)))
			{
				csmode(chaninfo, "+b %s", ban->mask);
				cskick(chan, nick->numeric, "Enforce: %s", ban->raison);
				return;
			}
		}

		if(CSetWelcome(chaninfo) && chaninfo->welcome[0] != '\0') /*si le welcome est activ ET que le welcome existe*/
		   csntc(nick, "Bienvenue sur \2%s\2 - %s", chaninfo->chan, chaninfo->welcome); /*on envoie la notice*/
	}
	/*On regarde l'access ici pour infoline, autoop et autovoice*/
	if(!nick->user || IsSuspend(nick->user)) return; /*ne va pas plus loin si l'user n'est pas auth ou qu'il est suspend (peut etre inutile)*/

	if(!(access=GetAccessIbyUserI(nick->user,chan)) || ASuspend(access)) return;/*ne va pas plus loin si l'user n'a pas d'access ou qu'il est suspendu*/
	access->lastseen = 1;

	if(is_burst) return;

	if(AOp(access) || AVoice(access))
	{
		new->is_voice = AVoice(access);
		new->is_op = CNoOps(chaninfo) ? 0 : AOp(access);

		csmode(chaninfo, "+%s%s %s %s", AOp(access) ? (!CNoOps(chaninfo) ? "o" : ""):"", AVoice(access) ? "v" : "", AOp(access) ? (!CNoOps(chaninfo) ? nick->numeric : "") : "" , AVoice(access) ? nick->numeric : "");
	}
	if(access->info) putserv("%s %s %s :[\2%s\2] %s\n", cs.num, TOKEN_PRIVMSG, chan, nick->user->nick, access->info);

	return;

}

void add_ban(aChan *cchan, const char *mask, const char *raison, const char *de, time_t debut, time_t fin, int level)
{
	aBan *new = calloc(1, sizeof(aBan)), *current;
	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour aBan %s sur %s !\n", mask, cchan->chan);
		return;
	}
	strncpy(new->mask, mask, 100);
	strncpy(new->raison, raison, 300);
	strncpy(new->de, de, NICKLEN);
	new->debut = debut;
	new->fin = fin;
	new->level = level;
	if(!cchan->banhead) cchan->banhead = new;
	else
	{
		for(current = cchan->banhead;current->next && match(current->mask, mask);current=current->next);
		if(!current->next) current->next=new;
		else free(new);
	}
}

void add_ignore(const char *host, time_t expiration)
{
	struct ignore *new = calloc(1, sizeof(struct ignore)), *current;

	if(!new)
	{
		putlog(LOG_ERREURS, "ERREUR MAJEURE: Impossible de malloc de la memoire pour un ignore %s !\n", host);
		return;
	}
	new->next = NULL;
	strncpy(new->host, host, HOSTLEN);
	new->exp = expiration;
	if(!ignorehead) ignorehead = new;
	else
	{
		for(current = ignorehead;current->next;current=current->next);
		if(!current->next) current->next=new;
		else free(new);
	}
}

