/* src/chanserv.c - Diverses commandes sur le module chanserv
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Gay@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ctype.h>
#include <main.h>

CVSINFO("$Id: chanserv.c,v 1.29 2003/05/03 16:51:46 romexzf Exp $");

extern const char *pasdeperm;

/*
 * show_access
 *
 * parv[0] = bot
 * parv[1] = chan
 * parv[2] = recherche
 */
void show_access(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aLink *lp;
	char *nick2 = parv[2], lev[4], comp = '\0';
	int i = 0, autoop, protect, suspend, autovoice;

	if(*nick2 == '>' || *nick2 == '<' || *nick2 == '=')
	{
		comp = *nick2;
		strncpy(lev, ++nick2, 4);
		if(parc > 2)
		{
			csntc(nick, "Ne pas mettre d'espace entre les caractres <,>,= et le niveau");
			return;
		}
		if(!is_num(lev))
		{
			csntc(nick, "La valeur du niveau doit etre numrique derrire < ou > ou =");
			return;
		}
	}
	autoop=getoption("-autoop",parv,parc,2,-1);
	autovoice=getoption("-autovoice",parv,parc,2,-1);
	protect=getoption("-protect",parv,parc,2,-1);
	suspend=getoption("-suspend",parv,parc,2,-1);
	csntc(nick, "------------------[ \0032\2%s\2\0036 > Liste des accs\3      ]------------------", chaninfo->chan);

	for(lp = chaninfo->access;lp;lp=lp->next)
	{
		if(!(lp->value.a->flag & A_WAITACCESS) && (
			 (comp!='\0' && ((comp=='>' && lp->value.a->level > atoi(lev))
			                 || (comp=='<' && lp->value.a->level < atoi(lev))
			                 || (comp=='=' && atoi(lev) == lp->value.a->level)))
			 /* || (*nick2=='*') apparament a pose problme */
			 || ( (!strchr(nick2, '*') || !strchr(nick2, '?'))
			      && !match(nick2, lp->value.a->user->nick))
			 || strstr(lp->value.a->user->nick, nick2))
			&& (!autoop || AOp(lp->value.a))
			&& (!autovoice || AVoice(lp->value.a))
			&& (!protect || AProtect(lp->value.a))
			&& (!suspend || ASuspend(lp->value.a)))
		{
			show_accessn(lp->value.a, lp->value.a->user, nick);
			csntc(nick, "-");
			i++;
		}
	}
	csntc(nick, "Un Total de %d entre%s trouve%s", i, i > 1 ? "s" : "", i > 1 ? "s": "");
	return;
}

void show_accessn(struct access *access, struct userinfo *user, aNick *num)
{
	char *auth = user->n ? "Oui" : "Non";

	csntc(num, "\2Username:\2 %s \2Niveau:\2 %d \2Identifi:\2 %s", user->nick, access->level, auth);
	csntc(num, "\2Options:\2%s", GetAccessOptions(access));
	csntc(num, "\2Dernire visite:\2 %s", access->lastseen == 1 ? "Actuellement dans le salon" : get_time(access->lastseen));


	if(access->info) csntc(num, "\2Infoline:\2 %s", access->info);
	return;
}

/*
 * add_user
 *
 * parv[1] = salon
 * parv[2] = username
 * parv[3] = niveau
 */
void add_user(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	anUser *u;
	anAccess *a;
	char *username = parv[2], *salon = parv[1]
#ifdef USE_MEMOSERV
,memo[MEMOLEN + 1]
#endif
;
	int niveau, flag = 0;

	if(!(u = ParseNickOrUser(nick, username))) return;

	if(!is_num(parv[3]) || (niveau = atoi(parv[3])) < 1 || niveau >= 500)
	{
		csntc(nick, "Le niveau doit tre un nombre entre 1 et 499");
		return;
	}
	if(!nick->user || (niveau >= ChanLevelbyUserI(nick->user, salon) && !IsAnAdmin(nick->user)))
	{
		csntc(nick, "Le niveau ne doit pas tre plus grand que le votre");
		return;
	}
	if(u->flag & U_PREJECT)
	{
		csntc(nick, "\2%s\2 refuse systmatiquement les propositions d'accs.", u->nick);
		return;
	}
	else if(u->flag & U_PACCEPT)
	{
		if(ChanLevelbyUserI(u, salon))
		{
			csntc(nick, "%s a dj un accs sur %s.", username, salon);
			return;
		}
		if(niveau >= 400) flag |= (A_OP | A_PROTECT);
		add_access(u, salon, niveau, flag, time(NULL));
		write_anuser(u);
		csntc(nick, "\2%s\2 a maintenant un accs sur \2%s\2 (%d)", u->nick, salon, niveau);
		if(u->n) csntc(u->n, "\2%s\2 vous a donn un accs sur \2%s\2 (%d)", nick->nick, salon, niveau);
#ifdef USE_MEMOSERV
		else
		{
			snprintf(memo, MEMOLEN, "\2%s\2 vous a donn un accs sur \2%s\2 (%d)", nick->nick, salon, niveau);
			add_memo(u, nick->user->nick, time(NULL), memo, u->n ? 1 : 0);
		}
#endif
		return;
	}
	else
	{
		for(a = u->accesshead;a;a = a->next)
		{
			if(!strcasecmp(salon, a->chan))
			{
				if((a->flag & A_WAITACCESS))	csntc(nick, "Vous avez dj propos un access  %s", username);
				else csntc(nick, "%s a dj un accs sur %s.", username, salon);
				return;
			}
		}

		add_access(u, salon, niveau, (niveau >= 400) ? (A_OP | A_PROTECT | A_WAITACCESS) : A_WAITACCESS, time(NULL));
		csntc(nick, "Vous avez bien propos  %s un access sur %s. Il va recevoir une notice ou un memo lui demandant de confirmer ou de refuser cet access", username, salon);

		if(u->n)
		{
			csntc(u->n, "%s vous a propos un access de %d sur %s.", nick->nick, niveau, salon);
			csntc(u->n, "Pour accepter tapez:\2 /%s %s ACCEPT %s \2, pour refuser tapez:\2 /%s %s REFUSE %s", cs.nick, RealCmd("myaccess"), salon, cs.nick, RealCmd("myaccess"), salon);
		}
#ifdef USE_MEMOSERV
		else
		{
			snprintf(memo, MEMOLEN, "%s vous a propos un access de %d sur %s. Pour accepter tapez:\2 /%s %s ACCEPT %s \2, pour refuser tapez:\2 /%s %s REFUSE %s", nick->nick, niveau, salon, cs.nick, RealCmd("myaccess"), salon, cs.nick, RealCmd("myaccess"), salon);
			add_memo(u, nick->user->nick, time(NULL), memo, u->n ? 1 : 0);
		}
#endif
		write_anuser(u);
		return;
	}
	return; /* on sait jamais :P */
}

/*
 * del_user
 *
 * parv[1] = #salon
 * parv[2] = username
 */
void del_user(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	anUser *u;
	anAccess *a;
	const char *salon = parv[1];
	int niveau, wait;

	if(!(u = ParseNickOrUser(nick, parv[2]))) return;

	for(a=u->accesshead;a && strcasecmp(a->chan, salon);a=a->next);
	if(!a || !(niveau = a->level))
	{
		csntc(nick, "%s n'a pas d'accs sur %s", u->nick, salon);
		return;
	}

	wait = (a->flag & A_WAITACCESS);
	if(ChanLevelbyUserI(nick->user, salon) <= niveau && !IsAnAdmin(nick->user))
	{
		csntc(nick, "Vous ne pouvez pas deluser un user d'un niveau suprieur ou gal au votre.");
		return;
	}
	if(niveau == 500)
	{
		csntc(nick, "Vous ne pouvez pas deluser l'accs 500 du salon ! Veuillez utiliser UNREG ou CHAN DEL pour supprimer le salon ou CHAN CHOWNER pour changer de propritaire.");
		return;
	}
	del_access(u, chaninfo);
	write_anuser(u);
	csntc(nick, "%s n'a plus %sd'accs sur %s", u->nick, wait ? "de proposition " : "", salon);
	if(u->n) csntc(u->n, "%s a supprim votre %saccs sur %s", nick->nick, wait ? "proposition d'" : "", salon);
	return;
}

void kick(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char r[150], *c = parv[1];
	aNick *targ;

	/* Verifications des protections de la cible. */
	if(!strcasecmp(cs.nick, parv[2]))
	{
		csntc(nick, "Je suis pas sur que a soit prfrable pour \2%s\2...", chaninfo->chan);
		return;
	}
	if(!(targ = getnickbynick(parv[2])))
	{
		csntc(nick, NOTCONNECTED, parv[2]);
		return;
	}
	if((targ->flag & N_SERVICE))
	{
		csntc(nick, "\2%s\2 est un Service. (+k)", targ->nick);
		return;
	}
	if(!(getjoininfo(targ, chaninfo->chan)))
	{
		csntc(nick, "\2%s\2 n'est pas sur %s", targ->nick, chaninfo->chan);
		return;
	}

	if(!check_protect(nick, targ, c)) return;

	if(parc < 3) strncpy(r, DEFRAISON, 150);
	else parv2msgn(parc, parv, 3, r, 150);

	cskick(c, targ->numeric, "(\2%s\2) %s", nick->user->nick, r);
	return;
}


void verify(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aNick *n;

	if(!strcasecmp(parv[1], cs.nick))
	{
		csntc(nick, "Yeah, c'est bien moi :)");
		return;
	}

	if(!strcasecmp(nick->nick, parv[1])) n = nick;
	else n = getnickbynick(parv[1]);
	if(!n)
	{
		csntc(nick, NOTCONNECTED, parv[1]);
		return;
	}

	if((n->flag & N_SERVICE))
	{
		csntc(nick, "%s est un Robot (\2Service\2).", n->nick);
		return;
	}

	if(!n->user)
	{
		csntc(nick, "%s n'est pas actuellement logu.", n->nick);
		return;
	}
	if(IsAnAdmin(nick->user) || n == nick) csntc(nick, "%s!%s@%s est logu sous l'username %s %s %s", n->nick, n->ident, n->host, n->user->nick, (n->user->level > 1) ? "- Administrateur des Services" : "", (n->flag & N_OPER) ? "- IRCop" : "");
	else csntc(nick, "%s est logu sous l'username %s %s %s", n->nick, n->user->nick, IsAnAdmin(n->user) ? "- Administrateur des Services" : "", (n->flag & N_OPER) ? " - IRCop" : "");
	return;
}

void mode(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	int a = 0;
	char *mode = parv[2];

	if(strchr(mode, 'o') || strchr(mode, 'b') || strchr(mode, 'v'))
	{
		csntc(nick, "Vous n'tes pas autoris  changer les Ops,Voices,Bans avec cette commande.");
		return;
	}
	if(!(nick->flag & N_OPER) && strchr(mode, 'O'))
	{
		csntc(nick, "Vous n'tes pas autoris  changer ce mode (O)");
		return;
	}
	if(strchr(mode, 'k')) a++;
	if(strchr(mode, 'l')) a++;
	csmode(chaninfo, "%s %s %s", mode, (a && parc > 2) ? parv[3] : "", (a > 1 && parc > 3) ? parv[4] : "");
	write_achan(chaninfo);
	return;
}

void topic(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	if(CLockTopic(chaninfo))
	{
		csntc(nick, "Le LOCKTOPIC est actif sur %s", parv[1]);
		return;
	}
	cstopic(chaninfo, "%s", parv2msg(parc, parv, 2, TOPICLEN));
	write_achan(chaninfo);
	return;
}

void rdeftopic(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	if(CLockTopic(chaninfo))
	{
		csntc(nick, "Le LOCKTOPIC est actif sur %s", parv[1]);
		return;
	}
	if(chaninfo->deftopic[0] != '\0') cstopic(chaninfo, "%s", chaninfo->deftopic);
	return;
}

void rdefmodes(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	csmode(chaninfo, "-%s %s", GetCModes(chaninfo), (chaninfo->flag & C_MKEY) ? chaninfo->key : "");
	if(chaninfo->defmodes[0] != '\0') csmode(chaninfo, "%s", chaninfo->defmodes);
	return;
}

void info(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aLink *lp;
	anAccess *access=NULL;
	int size;
	char *ptr;

	for(lp=chaninfo->access;lp && lp->value.a->user != nick->user;lp=lp->next);
	if(!lp) return;
	access = lp->value.a;

	if(parc < 2)
	{
		csntc(nick, "Veuillez prciser une infoline ou le mot cl 'none' pour la dfinir nulle.");
		return;
	}
	if(!strcasecmp(parv[2], "none"))
	{
		if(access->info)
		{
			free(access->info);
			access->info = NULL;
		}
		csntc(nick, "Infoline sur \2%s\2 supprime.", access->chan);
		return;
	}
	ptr = parv2msg(parc, parv, 2, 199);
	if(access->info) free(access->info);
	access->info = calloc(1, (size = (strlen(ptr) + 1)));
	strcpy(access->info, ptr);
	access->info[size-1] = 0;
	csntc(nick, "Infoline sur \2%s\2 change en: %s", access->chan, access->info);
	write_anuser(nick->user);
	return;
}

void invite(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aNick *n;
	aJoin *j;

	if(parc < 2) n = nick;
	else if (!(n = getnickbynick(parv[2])))
	{
		csntc(nick, NOTCONNECTED, parv[2]);
		return;
	}

	if((j=getjoininfo(n, chaninfo->chan)))
	{
		csntc(nick, "%s %s dj sur %s.", (n!=nick) ? n->nick : "Vous", (n!=nick) ? "est" : "tes", chaninfo->chan);
		return;
	}

	if(is_ban(n, chaninfo))
	{
		if(n==nick) csntc(nick, "Vous tes banni du salon %s : vous ne pouvez pas vous y inviter.", chaninfo->chan);
		else csntc(nick, "%s est banni du salon %s : vous ne pouvez l'y inviter", n->nick, chaninfo->chan);
		return;
	}
    putserv("%s %s %s :%s\n", cs.num, TOKEN_INVITE, n->nick, chaninfo->chan);
	return;
}

void clearmodes(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	csmode(chaninfo, "-%s %s", GetCModes(chaninfo), (chaninfo->flag & C_MKEY) ? chaninfo->key : "");
	csntc(nick, "Tous les modes de %s sont supprims.", parv[1]);
	return;
}

/*
 * see_alist
 *
 * parv[1] = chan
 */
void see_alist(aNick *nick, aChan *chan, int parc, char **parv)
{
	aLink *lp;

	csntc(nick, "------------------[ \0032\002%s\002\0036 > Liste des identifis\3 ]------------------", chan->chan);
	csntc(nick, "\2\037Pseudo Actuel\037 \037Username\037 \037Niveau\037");

	for(lp=chan->access;lp;lp=lp->next)
	{
		if(!lp->value.a->user->n) continue;
		csntc(nick, "%-13s %-8s %-6d", lp->value.a->user->n->nick, lp->value.a->user->nick, lp->value.a->level);
	}
}

void admin_say(aNick *nick, aChan *chaninfo, int parc, char **parv)
{

	putserv("%s %s %s :%s\n", cs.num, TOKEN_PRIVMSG, parv[1], parv2msg(parc, parv, 2, 400));
	return;
}

void admin_do(aNick *nick, aChan *chaninfo, int parc, char **parv)
{

	putserv("%s %s %s :\001ACTION %s\001\n", cs.num, TOKEN_PRIVMSG, parv[1], parv2msg(parc, parv, 2, 400));
	return;
}

